﻿using Abt.Controls.SciChart;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SciChartVirtualization
{
    /// <summary>
    /// Interaction logic for VirtualizationControl.xaml
    /// </summary>
    public partial class VirtualizationControl : UserControl
    {
        DataProvider dataProvider = new DataProvider();

        DateTime rangeBegin; 
        DateTime rangeEnd;
        double rangeGranularitySeconds;

        DateTime minDate = new DateTime(2013, 01, 01);
        DateTime maxDate = new DateTime(2013, 12, 31);

        DataSeriesSet<DateTime, double> dataSeriesSet;
        IXyDataSeries<DateTime, double> series;

        public VirtualizationControl()
        {
            InitializeComponent();
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {

            dataSeriesSet = new DataSeriesSet<DateTime, double>();
            sciChart.DataSet = dataSeriesSet;

            series = dataSeriesSet.AddSeries();

            UpdateData(new DateTime(2013,01,01), new DateTime(2013,12,31));

        }

        private void DateTimeAxis_VisibleRangeChanged(object sender, VisibleRangeChangedEventArgs e)
        {
            DateTime begin = (DateTime)e.NewVisibleRange.Min;
            DateTime end = (DateTime)e.NewVisibleRange.Max;
            
            UpdateData(begin, end);
        }

        private void UpdateData(DateTime begin, DateTime end)
        {

            TimeSpan len = end - begin;

            double newGran = len.TotalSeconds / 1000;
            newGran = Math.Max(newGran, 0.001);

            if (rangeBegin <= begin && rangeEnd >= end && rangeGranularitySeconds <= newGran*2 && rangeGranularitySeconds >= newGran/2)
                return;

            rangeBegin = begin - len;
            rangeEnd = end + len;
            rangeGranularitySeconds = newGran;

            if (rangeBegin < minDate)
                rangeBegin = minDate;

            if (rangeEnd > maxDate)
                rangeEnd = maxDate;

            using (dataSeriesSet.SuspendUpdates())
            {
                series.Clear();
                dataProvider.GetData(rangeBegin, rangeEnd, TimeSpan.FromSeconds(rangeGranularitySeconds), series);
            }
        }
    }

    class DataProvider
    {

        public void GetData(DateTime begin, DateTime end, TimeSpan granularity, IXyDataSeries<DateTime, double> series)
        {

            for (DateTime d = begin; d <= end; d = d + granularity)
            {
                 series.Append(d, GetValue(d));
            }

        }


        private double GetValue(DateTime d)
        {
            
            double t = d.Ticks / 4e13;

            double val = 0;
            double f = 1;

            for (int i = 1; i <25; i++)
            {
                val += Math.Sin(t*f)/f;
                f*=2;
            }

            return val;

        }

    }
}
